==> immersiveoverlays-1.4.1%2B1.21.8-fabric-sources.jar: cc/cassian/immersiveoverlays/overlay/BiomeOverlay.java <==
package cc.cassian.immersiveoverlays.overlay;

import cc.cassian.immersiveoverlays.ModClient;
import cc.cassian.immersiveoverlays.config.ModConfig;
import org.apache.commons.lang3.text.WordUtils;

import java.util.Objects;
import net.minecraft.class_1074;
import net.minecraft.class_1959;
import net.minecraft.class_1972;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_6880;
import net.minecraft.class_746;

public class BiomeOverlay {
    public static boolean showBiome = false;
    public static class_2960 UNDEFINED = ModClient.locate("textures/immersiveoverlays/undefined.png");


    //? if >1.20 {
    public static void renderGameOverlayEvent(class_332 guiGraphics) {
    //?} else {
        /*public static void renderGameOverlayEvent(PoseStack guiGraphics) {
     *///?}
        if (!showBiome || !ModConfig.get().biome_enable)
            return;
        if (ModConfig.get().biome_reduced_info) {
            return;
        }
        var mc = class_310.method_1551();
        if (OverlayHelpers.shouldCancelRender(mc))
            return;

        if (mc.field_1724 == null) return;
        class_6880<class_1959> biome = getBiome(mc.field_1724);
        class_2960 biomeId = getId(biome);
        String biomeText = formatBiome(biomeId);

        int yPlacement = ModConfig.get().biome_vertical_position;
        int textYPlacement = yPlacement+2;
        int xOffset = 3;
        // The amount of offset needed to display the biome icons.
        int iconXOffset = 0;
        class_2960 sprite = UNDEFINED;
        if (ModConfig.get().biome_icons) {
            sprite = getBiomeSprite(biomeId, true);
            if (!sprite.equals(UNDEFINED)) {
                iconXOffset = 20;
            }
        }
        int tooltipSize = 21;

        int fontWidth = mc.field_1772.method_1727(biomeText)+iconXOffset;

        if (ModConfig.get().avoid_overlapping) {
            if (!(ClockOverlay.showTime || ClockOverlay.showWeather) || !ModConfig.get().clock_enable) {
                yPlacement = yPlacement - 24;
                textYPlacement = textYPlacement - 24;
            }
            if (ClockOverlay.shouldShowSeasons()) {
                yPlacement += 15;
                textYPlacement += 15;
            }
        }
        if (OverlayHelpers.playerHasPotions(mc.field_1724, ModConfig.get().biome_horizontal_position_left)) {
            yPlacement += OverlayHelpers.moveBy(mc.field_1724);
            textYPlacement += OverlayHelpers.moveBy(mc.field_1724);
        }

        int windowWidth = mc.method_22683().method_4486();
        int xPlacement = OverlayHelpers.getPlacement(windowWidth, fontWidth, ModConfig.get().biome_horizontal_position_left);
        // render text
        if (showBiome) {
            OverlayHelpers.renderBackground(guiGraphics, windowWidth, fontWidth, xPlacement, xOffset, yPlacement, tooltipSize, ModConfig.get().biome_horizontal_position_left);
            OverlayHelpers.drawString(guiGraphics, mc.field_1772, biomeText, xPlacement-xOffset+iconXOffset, textYPlacement, getTextColour(biome));
            if (ModConfig.get().biome_icons && !sprite.equals(UNDEFINED))
                OverlayHelpers.blitSprite(guiGraphics, sprite, xPlacement-xOffset-1, textYPlacement-4);
        }
    }

    private static Integer getTextColour(class_6880<class_1959> biome) {
        if (ModConfig.get().biome_text_tinted)
            return TemperatureOverlay.getBiomeTemperature(biome).color();
        return ModConfig.get().biome_text_colour;
    }

    public static class_2960 getBiomeSprite(class_2960 biome, boolean allowRedirect) {
        var manager = class_310.method_1551().method_1478();
        var path = "textures/immersiveoverlays/"+ biome.method_12832();
        var key = class_2960.method_43902(biome.method_12836(), "%s.png".formatted(path));
        var redirect = class_2960.method_43902(biome.method_12836(), "%s.txt".formatted(path));
        if (manager.method_14486(key).isPresent())
            return key;
        else {
            if (manager.method_14486(redirect).isPresent() && allowRedirect) {
                try {
                    return getBiomeSprite(Objects.requireNonNull(class_2960.method_12829(manager.openAsReader(redirect).lines().toList().get(0))), false);
                } catch (Exception e) {
                    return UNDEFINED;
                }
            } else {
                return UNDEFINED;
            }
        }
    }

    public static class_6880<class_1959> getBiome(class_746 player) {
        return player.method_37908
        //? if >1.20 {
        ()
        //?}
        .method_23753(player.method_24515());
    }

    public static class_2960 getId(class_6880<class_1959> biome) {
        return biome.method_40230().orElse(class_1972.field_9473).method_29177();
    }

    public static String formatBiome(class_2960 biome) {
        var path = biome.method_12832();
        var namespace = biome.method_12836();
        var key = "biome.%s.%s".formatted(namespace, path);
        if (class_1074.method_4663(key)) {
            return class_1074.method_4662(key);
        } else return WordUtils.capitalize(path);
    }

}

==> immersiveoverlays-1.4.2%2B1.19.2-fabric-sources.jar: cc/cassian/immersiveoverlays/overlay/BiomeOverlay.java <==
package cc.cassian.immersiveoverlays.overlay;

import cc.cassian.immersiveoverlays.ModClient;
import cc.cassian.immersiveoverlays.config.ModConfig;
import com.mojang.blaze3d.systems.RenderSystem;
import org.apache.commons.lang3.text.WordUtils;

import java.io.IOException;
import java.util.Objects;
import net.minecraft.class_1074;
import net.minecraft.class_1959;
import net.minecraft.class_1972;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_4587;
import net.minecraft.class_6880;
import net.minecraft.class_746;

public class BiomeOverlay {
    public static boolean showBiome = false;
    public static class_2960 UNDEFINED = ModClient.locate("textures/immersiveoverlays/undefined.png");


    //? if >1.20 {
    /*public static void renderGameOverlayEvent(GuiGraphics guiGraphics) {
    *///?} else {
        public static void renderGameOverlayEvent(class_4587 guiGraphics) {
     //?}
        if (!showBiome || !ModConfig.get().biome_enable)
            return;
        if (ModConfig.get().biome_reduced_info) {
            return;
        }
        var mc = class_310.method_1551();
        if (OverlayHelpers.shouldCancelRender(mc))
            return;

        if (mc.field_1724 == null) return;
        class_6880<class_1959> biome = getBiome(mc.field_1724);
        class_2960 biomeId = getId(biome);
        String biomeText = formatBiome(biomeId);

        int yPlacement = ModConfig.get().biome_vertical_position;
        int textYPlacement = yPlacement+2;
        int xOffset = 3;
        // The amount of offset needed to display the biome icons.
        int iconXOffset = 0;
        class_2960 sprite = UNDEFINED;
        if (ModConfig.get().biome_icons) {
            sprite = getBiomeSprite(biomeId, true);
            if (!sprite.equals(UNDEFINED)) {
                iconXOffset = 20;
            }
        }
        int tooltipSize = 21;

        int fontWidth = mc.field_1772.method_1727(biomeText)+iconXOffset;

        if (ModConfig.get().avoid_overlapping) {
            if (!(ClockOverlay.showTime || ClockOverlay.showWeather) || !ModConfig.get().clock_enable) {
                yPlacement = yPlacement - 24;
                textYPlacement = textYPlacement - 24;
            }
            if (ClockOverlay.shouldShowSeasons()) {
                yPlacement += 15;
                textYPlacement += 15;
            }
        }
        if (OverlayHelpers.playerHasPotions(mc.field_1724, ModConfig.get().biome_horizontal_position_left)) {
            yPlacement += OverlayHelpers.moveBy(mc.field_1724);
            textYPlacement += OverlayHelpers.moveBy(mc.field_1724);
        }

        int windowWidth = mc.method_22683().method_4486();
        int xPlacement = OverlayHelpers.getPlacement(windowWidth, fontWidth, ModConfig.get().biome_horizontal_position_left);
        // render text
        if (showBiome) {
            OverlayHelpers.renderBackground(guiGraphics, windowWidth, fontWidth, xPlacement, xOffset, yPlacement, tooltipSize, ModConfig.get().biome_horizontal_position_left);
            OverlayHelpers.drawString(guiGraphics, mc.field_1772, biomeText, xPlacement-xOffset+iconXOffset, textYPlacement, getTextColour(biome));
            if (ModConfig.get().biome_icons && !sprite.equals(UNDEFINED))
                OverlayHelpers.blitSprite(guiGraphics, sprite, xPlacement-xOffset-1, textYPlacement-4);
        }
    }

    private static Integer getTextColour(class_6880<class_1959> biome) {
        if (ModConfig.get().biome_text_tinted)
            return TemperatureOverlay.getBiomeTemperature(biome).color();
        return ModConfig.get().biome_text_colour;
    }

    public static class_2960 getBiomeSprite(class_2960 biome, boolean allowRedirect) {
        var manager = class_310.method_1551().method_1478();
        var path = "textures/immersiveoverlays/"+ biome.method_12832();
        var key = ModClient.locate(biome.method_12836(), "%s.png".formatted(path));
        var redirect = ModClient.locate(biome.method_12836(), "%s.txt".formatted(path));
        //? if >1.19 {
        if (manager.method_14486(key).isPresent())
            return key;
        else {
            if (manager.method_14486(redirect).isPresent() && allowRedirect) {
                try {
                    return getBiomeSprite(Objects.requireNonNull(class_2960.method_12829(manager.openAsReader(redirect).lines().toList().get(0))), false);
                } catch (Exception e) {
                    return UNDEFINED;
                }
            } else {
                return UNDEFINED;
            }
        }
        //?} else {
        /*if (allowRedirect) {
            try {
                return getBiomeSprite(Objects.requireNonNull(ResourceLocation.tryParse(manager.getResource(redirect).getInputStream().toString())), false);
            } catch (Exception e) {
                return UNDEFINED;
            }
        } else {
            return UNDEFINED;
        }
        *///?}
    }

    public static class_6880<class_1959> getBiome(class_746 player) {
        return player.field_6002
        //? if >1.20 {
        /*()
        *///?}
        .method_23753(player.method_24515());
    }

    public static class_2960 getId(class_6880<class_1959> biome) {
        return biome.method_40230().orElse(class_1972.field_9473).method_29177();
    }

    public static String formatBiome(class_2960 biome) {
        var path = biome.method_12832();
        var namespace = biome.method_12836();
        var key = "biome.%s.%s".formatted(namespace, path);
        if (class_1074.method_4663(key)) {
            return class_1074.method_4662(key);
        } else return WordUtils.capitalize(path.replace("_"," "));
    }

}
